// -*- C++ -*-
/*!
 * @file  OmniTo2WD.cpp
 * @brief omni-directional motion by 2WD base
 * @date $Date$
 *
 * @author Toshiba
 *
 * $Id$
 */

/*
 * Copyright (c) 2011, Toshiba Corporation.
 * This program is made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 */


#include "OmniTo2WD.h"
#define RAD_TO_DEG 180/M_PI
#define TIMEOUT_ODM 500
#define UNIT_TEST 1

///////// add 2011/09/29 Toshiba ////////
enum{
  FINISH = 0,
  ANGLE_CHECK,
  TURNING,
  TURNING2,
  GOAL_CHECK,
  SPEED_DOWN,
  STOPPING
};
#define DEG_TO_RAD M_PI/180.0
#define PATH_START 100
#define PATH_SUBGOAL 101
#define PATH_END 102
#define TOLERANCE 0.05 //max error between odometry position and commanded position, unit:mm, 0.05 derived from Navigation.cpp
#define TOLERANCE_ROT (2*DEG_TO_RAD)//
///////// add 2011/09/29 Toshiba ////////max error between odometry orientation and commanded orientation, unit:rad


// Module specification
// <rtc-template block="module_spec">
static const char* omnito2wd_spec[] =
  {
    "implementation_id", "OmniTo2WD",
    "type_name",         "OmniTo2WD",
    "description",       "omni-directional motion by 2WD base",
    "version",           "0.0.1",
    "vendor",            "TOSHIBA",
    "category",          "Controller",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
OmniTo2WD::OmniTo2WD(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_current_positionIn("current_position", m_current_position),
    m_goalIn("goal", m_goal),
    m_min_pathOut("min_path", m_min_path),
		// add:s 2011.09.28 by Toshiba
		m_statusOut("status", m_status),
		m_statusGetIn("statusGet", m_statusGet)
		// add:e 2011.09.28 by Toshiba

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
OmniTo2WD::~OmniTo2WD()
{
}


/*!
 * set Configuration parameters
 */
RTC::ReturnCode_t OmniTo2WD::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("current_position", m_current_positionIn);
  addInPort("goal", m_goalIn);
  // add:s 2011.09.28 by Toshiba
  addInPort("statusGet", m_statusGetIn);
  // add:e 2010.09.28 by Toshiba
  
  // Set OutPort buffer
  addOutPort("min_path", m_min_pathOut);
  // add:s 2011.09.28 by Toshiba
  registerOutPort("status", m_statusOut);
  // add:e 2010.09.28 by Toshiba

  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>
//  /////for debug
//	TimedPath2DSeq out_path;
//	std::vector<IIS::TimedPose2D> in_path(4);
//
//	in_path[0].data.position.x=0.0;
//	in_path[0].data.position.y=0.0;
//	in_path[0].data.heading=0.0;
//
//	in_path[1].data.position.x=-0.5;
//	in_path[1].data.position.y=0.0;
//	in_path[1].data.heading=0.0;//-90*DEG_TO_RAD;
//
//	in_path[2].data.position.x=-0.5;
//	in_path[2].data.position.y=-0.5;
//	in_path[2].data.heading=0.0;
//
//	in_path[3].data.position.x=-1.0;
//	in_path[3].data.position.y=-1.0;
//	in_path[3].data.heading=0.0;
//
//  this->calc2WDpath(in_path, &out_path);
//  //print results
//  for(unsigned int i=0; i<out_path.pose.length(); i++){
//  	std::cout<<out_path.pose[i].position.x<<","<<out_path.pose[i].position.y<<
//  			","<<out_path.pose[i].heading<<std::endl;
//  }
  //////// for debug
//	//calculates newTheta
//	double dx=1.0-1.0144;
//	double dy=0.0-0.0113;
//	std::cout<<this->calcDirection(dx,dy)<<std::endl;


  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t OmniTo2WD::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OmniTo2WD::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OmniTo2WD::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 */

RTC::ReturnCode_t OmniTo2WD::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*!
 * (1)close files for debugging
 */

RTC::ReturnCode_t OmniTo2WD::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*!
 */

RTC::ReturnCode_t OmniTo2WD::onExecute(RTC::UniqueId ec_id)
{
	static bool pathFlag=false;//path or goal
	static std::vector<IIS::TimedPose2D> in_path;
	if(this->m_goalIn.isNew()){
		this->m_goalIn.read();
		//if path is coming, read the path
		if(this->m_goal.id.length()>0){
			if(this->m_goal.id[0]==PATH_START){
				//if(pathFlag==false){
					pathFlag=true;
					in_path.clear();
				//}
				in_path.push_back(this->m_goal);
				return RTC::RTC_OK;
			}
			else if(this->m_goal.id[0]==PATH_SUBGOAL){
				in_path.push_back(this->m_goal);
				return RTC::RTC_OK;
			}
			else if(this->m_goal.id[0]==PATH_END){
				in_path.push_back(this->m_goal);
			}
		}
		else{	}//goal or end of path

		//get odometory
		int cntOdm=0;
		do{
			if(this->m_current_positionIn.isNew()){
				this->m_current_positionIn.read();
				break;
			}
			//while(!m_current_positionIn.isEmpty())m_current_positionIn.read();
			cntOdm++;
		}while(cntOdm<TIMEOUT_ODM);
		if(cntOdm>=TIMEOUT_ODM){
#if UNIT_TEST
			this->m_current_position.data.position.x=0.0;
			this->m_current_position.data.position.y=0.0;
			this->m_current_position.data.heading=0.0;
#else
			std::cerr<<"Odometry data does not come!"<<std::endl;
			return RTC::RTC_ERROR;
#endif
		}

		if(pathFlag){
			//insert start position
			in_path.insert(in_path.begin(), this->m_current_position);
			/////for debug
			std::cout<<"----------in_path-----------"<<std::endl;
			  for(std::vector<IIS::TimedPose2D>::iterator p=in_path.begin();
			  		p!=in_path.end(); p++){
			  	std::cout<<p->data.position.x<<","<<
			  			p->data.position.y<<","<<
			  			p->data.heading<<","<<std::endl;
			  }
			/////////////
			this->calc2WDpath(in_path, &this->m_min_path);
			pathFlag=false;
			/////for debug
			std::cout<<"----------out_path-----------"<<std::endl;
			  for(unsigned int i=0; i<this->m_min_path.pose.length(); i++){
			  	std::cout<<this->m_min_path.pose[i].position.x<<","<<this->m_min_path.pose[i].position.y<<
			  			","<<this->m_min_path.pose[i].heading<<std::endl;
			  }
			 /////////////

		}
		else{
			this->calc2WDpath(this->m_current_position, this->m_goal, &this->m_min_path);
		}
		this->m_min_pathOut.write();

	  //////// add 20110929 Toshiba ///////
	  //wait move end
		this->m_statusGet.data=ANGLE_CHECK;
		int cntIsNewNavi=0;//for timeout
	  do{
	  	//wait isNew
	  	if(this->m_statusGetIn.isNew()){
	  		cntIsNewNavi=0;
	  		//read
	  		this->m_statusGetIn.read();
	  		//write
	  		this->m_status.data=this->m_statusGet.data;
	  		this->m_statusOut.write();
	  	}
	  	else{cntIsNewNavi++;}
	  }while(this->m_statusGet.data!=FINISH && cntIsNewNavi<10000);
	  if(this->m_statusGet.data==FINISH){std::cout<<"finish!"<<std::endl;}
	  if(cntIsNewNavi>=10000){std::cerr<<"status does not come!"<<std::endl;}
	  //////// add 20110929 Toshiba ///////

		return RTC::RTC_OK;
	}
	else{ return RTC::RTC_OK; }
}

/*
RTC::ReturnCode_t OmniTo2WD::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OmniTo2WD::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OmniTo2WD::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OmniTo2WD::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t OmniTo2WD::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

//calculate a path for 2WD robot base
//input start(x1,y1,theta1)
//input goal(x2,y2,theta2)
//output out_path(start, rotation, forward motion, rotation(=goal))
//out_path is absolute value(similar to world coordinate system)
///// pathLength was modified by Toshiba at 20111004
int OmniTo2WD::calc2WDpath(IIS::TimedPose2D start, IIS::TimedPose2D goal, IIS::TimedPath2DSeq *out_path){
	//calculates newTheta
	double dx=goal.data.position.x-start.data.position.x;
	double dy=goal.data.position.y-start.data.position.y;
	double dist=sqrt( dx*dx + dy*dy );
	int pathLength;
	double newTheta=0.0;
	//if dist is too small, only rotation
	if(dist<=TOLERANCE){	pathLength=2;}
	else{
		newTheta=this->calcDirection(dx,dy);
		/////debug
		std::cout<<"newTheta:"<<newTheta*RAD_TO_DEG<<std::endl;
		if( fabs(newTheta-start.data.heading)<TOLERANCE_ROT ){
			//comment out because Navigation ignores 2-path move without rotation(ex, (0,0,0) to (1.0,0))
//			if(fabs(newTheta-goal.data.heading)<TOLERANCE_ROT ){pathLength=2;}
//			else{ pathLength=3; }

			pathLength=3;
		}
		else{ pathLength=4; }
	}

	//initialize out_path
  out_path->id.length(pathLength);
  out_path->pose.length(pathLength);
  out_path->velocity.length(pathLength);
  out_path->error.length(pathLength);
  for(int i=0; i<pathLength; i++){
		out_path->id[i] = i;
		out_path->velocity[i].vx = 1.0;
		out_path->velocity[i].vy = 0.0;
		out_path->velocity[i].va = 0.0;
		out_path->error[i] = 0.0;
  }

  ////input start pose
	out_path->pose[0].position.x = start.data.position.x;
	out_path->pose[0].position.y = start.data.position.y;
	out_path->pose[0].heading = start.data.heading;

	if(pathLength==2){
		////rotation(goal)
		out_path->pose[1].position.x = goal.data.position.x;
		out_path->pose[1].position.y = goal.data.position.y;
		out_path->pose[1].heading = goal.data.heading;
	}
	else if(pathLength==3){
		////forward motion
		out_path->pose[1].position.x = goal.data.position.x;
		out_path->pose[1].position.y = goal.data.position.y;
		out_path->pose[1].heading = newTheta;
		////rotation(goal)
		out_path->pose[2].position.x = goal.data.position.x;
		out_path->pose[2].position.y = goal.data.position.y;
		out_path->pose[2].heading = goal.data.heading;
	}
	else if(pathLength==4){
		////rotation
		out_path->pose[1].position.x = start.data.position.x;
		out_path->pose[1].position.y = start.data.position.y;
		out_path->pose[1].heading = newTheta;
		////forward motion
		out_path->pose[2].position.x = goal.data.position.x;
		out_path->pose[2].position.y = goal.data.position.y;
		out_path->pose[2].heading = newTheta;
		////rotation(goal)
		out_path->pose[3].position.x = goal.data.position.x;
		out_path->pose[3].position.y = goal.data.position.y;
		out_path->pose[3].heading = goal.data.heading;
	}
	else{
		std::cerr<<"invalid pathLength!!"<<std::endl;
		return -1;
	}

	//////debug
	std::cout<<"out_path(x,y,theta):"<<std::endl;
	for(int i=0; i<pathLength; i++){
		std::cout<<out_path->pose[i].position.x<<"\t"<<out_path->pose[i].position.y<<"\t"
				<<out_path->pose[i].heading<<std::endl;
	}

	return 0;
}


//calculate path using path
//input in_path(x1,y1,theta1), ...,(x_n,y_n,theta_n). (x1,y1,theta1) is now pose
//output out_path(start, rotation, forward motion, rotation, forward motion, ... ,rotation(=goal))
//out_path is absolute value(similar to world coordinate system)
int OmniTo2WD::calc2WDpath(std::vector<IIS::TimedPose2D> in_path, TimedPath2DSeq *out_path){
	//calculate subgoal
	std::vector<TimedPath2DSeq> tmp_out_path_vec(0);
	for(std::vector<IIS::TimedPose2D>::iterator p=in_path.begin(); p!=in_path.end()-1; p++){
		TimedPath2DSeq tmp_out_path;
		this->calc2WDpath(*p, *(p+1), &tmp_out_path);
		tmp_out_path_vec.push_back(tmp_out_path);
	}

	//connect subgoal
	//create out_path
	int pathLength=tmp_out_path_vec.size()*2+2;
  out_path->id.length(pathLength);
  out_path->pose.length(pathLength);
  out_path->velocity.length(pathLength);
  out_path->error.length(pathLength);

  for(int i=0; i<pathLength; i++){
		out_path->id[i] = i;
		out_path->velocity[i].vx = 1.0;
		out_path->velocity[i].vy = 0.0;
		out_path->velocity[i].va = 0.0;
		out_path->error[i] = 0.0;
  }
  ////start
	out_path->pose[0] = in_path[0].data;
	//out_path->pose[0].position.x = in_path[0].data.position.x;
	//out_path->pose[0].position.y = in_path[0].data.position.y;
	//out_path->pose[0].heading = in_path[0].data.heading;
	////subgoal
	int pathCnt=1;
	for(std::vector<TimedPath2DSeq>::iterator p=tmp_out_path_vec.begin(); p!=tmp_out_path_vec.end(); p++){
		for(unsigned int i=1; i<p->pose.length()-1; i++, pathCnt++){
			out_path->pose[pathCnt] =p->pose[i];
		}
	}
	////goal
	out_path->pose[pathCnt]=in_path[in_path.size()-1].data;

	return 0;
}


double OmniTo2WD::calcDirection(double dx, double dy){
	//std::cout<<"dx:"<<dx<<"dy:"<<dy<<std::endl;
	//if(fabs(dx)<0.02 && fabs(dy)<0.02){return 0.0;}//under 2cm, no rotation<--this no rotation causes vibration of base
	if(sqrt(dx*dx+dy*dy) <TOLERANCE){return 0.0;}
	//calc atan2
  double newTheta=atan2(dy,dx);

	return newTheta;
}



extern "C"
{
 
  void OmniTo2WDInit(RTC::Manager* manager)
  {
    coil::Properties profile(omnito2wd_spec);
    manager->registerFactory(profile,
                             RTC::Create<OmniTo2WD>,
                             RTC::Delete<OmniTo2WD>);
  }
  
};


