// -*- C++ -*-
/*!
 * @file  OmniTo2WD.h
 * @brief omni-directional motion by 2WD base
 * @date  $Date$
 *
 * @author Toshiba
 *
 * $Id$
 */

/*
 * Copyright (c) 2011, Toshiba Corporation.
 * This program is made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 */


#ifndef OMNITO2WD_H
#define OMNITO2WD_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>
//#include <rtm/idl/IIS.h>

//---[add]--------------------------------
#include "intellirobotStub.h"
#include "intellirobotSkel.h"
#include "intellirobot.hh"

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>
#include <math.h>
#include <vector>

using namespace RTC;
using namespace IIS;

/*!
 * @class OmniTo2WD
 * @brief omni-directional motion by 2WD base
 *
 * This component converts a goal for a omni-directional mobile robot base to a 
 * path for 2WD robot base, such as 
 * a set of rotation, forward-motion, and rotation. 
 *
 * //Inport
 * current_position:current pose of a robot base.
 * //Outport
 *
 * -PathPlanning RT component
 * in 
 * http://210.154.184.16/pukiwiki/?SYS_001_V100
 *
 */
class OmniTo2WD
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  OmniTo2WD(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~OmniTo2WD();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * set Configuration parameters
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre none
   * @post none
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * (1) update INPORT's data
   * (2) open files for debugging
   * (3) initialize
   * (4) get robot's parameter
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre none
   * @post none
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * (1)close files for debugging
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre none
   * @post none
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * (1)getting current localized position from LocalizationRTC
   * (2)calculating Target Trans/Rot velocity values for controlling robot's velocity
   * (3)sending target Trans/Rot Velocity data to MotorControlRTC
   * (4)sending distance data between current position and target point to LocalPathP
   * lanningRTC
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre none
   * @post none
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  IIS::TimedPose2D m_current_position;
  /*!
   * current pose of the center of a base
   * - Number: pose:1 ,  timeStamp:1
   * - Unit: pose : [mm][rad], timeStamp: [sec],[nanosec]
   */
  InPort<IIS::TimedPose2D> m_current_positionIn;
  IIS::TimedPose2D m_goal;
  /*!
   * goal of the center of a robot base
   * - Unit: [m],[rad]
   */
  InPort<IIS::TimedPose2D> m_goalIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedPath2DSeq m_min_path;
  /*!
   * a path of the center of a base
   * - Semantics: 2WD robot moves omni-direction by the following path:
   *              1. rotation
   *              2. forward motion
   *              3. rotation
   * - Unit: path:[m][deg],  timeStamp: [sec],[nanosec]
   */
  OutPort<TimedPath2DSeq> m_min_pathOut;

  // add:s 2011.09.28 by Toshiba
  RTC::TimedState m_status;
  OutPort<RTC::TimedState> m_statusOut;
  RTC::TimedState m_statusGet;
  InPort<RTC::TimedState> m_statusGetIn;
  // add:e 2011.09.28 by Toshiba

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  int calc2WDpath(IIS::TimedPose2D start, IIS::TimedPose2D goal, IIS::TimedPath2DSeq *out_path);
  int calc2WDpath(std::vector<IIS::TimedPose2D> in_path, TimedPath2DSeq *out_path);
  double calcDirection(double dx, double dy);
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void OmniTo2WDInit(RTC::Manager* manager);
};

#endif // OMNITO2WD_H
