// -*- C++ -*-
/*!
 * @file  PositionInput.cpp * @brief compornent * $Date$ 
 *
 * $Id$ 
 */
/*
 * Copyright (c) 2011, Toshiba Corporation.
 * This program is made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 */


#include "PositionInput.h"

// Module specification
// <rtc-template block="module_spec">
static const char* positioninput_spec[] =
  {
    "implementation_id", "PositionInput",
    "type_name",         "PositionInput",
    "description",       "compornent",
    "version",           "1.0",
    "vendor",            "MyName",
    "category",          "example",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_x", "3.4",
    "conf.default.02_y", "7.7",
    "conf.default.03_th", "-1.57",
    ""
  };
// </rtc-template>

///////// add 2011/09/29 Toshiba ////////
enum{
  FINISH = 0,
  ANGLE_CHECK,
  TURNING,
  TURNING2,
  GOAL_CHECK,
  SPEED_DOWN,
  STOPPING
};
#define DEG_TO_RAD M_PI/180.0
#define PATH_START 100
#define PATH_SUBGOAL 101
#define PATH_END 102
///////// add 2011/09/29 Toshiba ////////

PositionInput::PositionInput(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_positionOut("position", m_position),
    // add:s 2011.09.28 by Toshiba
    m_statusGetIn("statusGet", m_statusGet)
	// add:e 2011.09.28 by Toshiba

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  // add:s 2011.09.28 by Toshiba
  addInPort("statusGet", m_statusGetIn);
  // add:e 2010.09.28 by Toshiba

  // Set OutPort buffer
  registerOutPort("position", m_positionOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

}

PositionInput::~PositionInput()
{
}


RTC::ReturnCode_t PositionInput::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("01_x", m_01_x, "3.4");
  bindParameter("02_y", m_02_y, "7.7");
  bindParameter("03_th", m_03_th, "-1.57");
  // </rtc-template>

  ///// add 20110930 Toshiba
  IIS::TimedPose2D p1;
  p1.id.length(1);
  p1.data.position.x=0.2;
  p1.data.position.y=0.0;
  p1.data.heading=0.0;
  p1.id[0]=PATH_START;
  defined_path.push_back(p1);

  p1.data.position.x=0.2;
  p1.data.position.y=-0.2;
  p1.data.heading=-90*DEG_TO_RAD;
  p1.id[0]=PATH_SUBGOAL;
  defined_path.push_back(p1);

//  p1.data.position.x=-0.2;
//  p1.data.position.y=0.2;
//  p1.data.heading=90*DEG_TO_RAD;
//  p1.id[0]=PATH_SUBGOAL;
//  defined_path.push_back(p1);

  p1.data.position.x=0.0;
  p1.data.position.y=0.0;
  p1.data.heading=0.0;
  p1.id[0]=PATH_END;
  defined_path.push_back(p1);
  ////////////

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t PositionInput::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t PositionInput::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t PositionInput::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t PositionInput::onActivated(RTC::UniqueId ec_id)
{
//  m_position.data.position.x = m_01_x;
//  m_position.data.position.y = m_02_y;
//  m_position.data.heading    = m_03_th;
//  RTC_INFO(("input goal position"))
//  RTC_INFO(("%lf %lf %lf",m_position.data.position.x, m_position.data.position.y, m_position.data.heading))
//  m_positionOut.write();
//  std::cout<<"output "<<m_position.data.position.x<<","<<m_position.data.position.y
//  		<<","<<m_position.data.heading<<std::endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PositionInput::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t PositionInput::onExecute(RTC::UniqueId ec_id)
{
  //////// modified 20110929 Toshiba ///////
	int goalOrPath=0;
	printf("input goal or path. goal:1, path:2");
	std::cin>>goalOrPath;

	if(goalOrPath==2){//path
		for(std::vector<IIS::TimedPose2D>::iterator p=this->defined_path.begin();
				p!=this->defined_path.end(); p++){
			this->m_position=*p;
			m_positionOut.write();
			sleep(0.5);
		}
	}
	else{//goal
		printf("input position(x y theta)>");
		std::cout<<std::endl<<"x[m]=";std::cin>>m_position.data.position.x;
		std::cout<<std::endl<<"y[m]=";std::cin>>m_position.data.position.y;
		std::cout<<std::endl<<"theta[rad]=";std::cin>>m_position.data.heading;

		std::cout<<"result:"<<m_position.data.position.x<<","<<m_position.data.position.y<<","<<m_position.data.heading<<std::endl;
	 // scanf("%lf %lf %lf",&m_position.data.position.x,&m_position.data.position.y,&m_position.data.heading);
		m_positionOut.write();
	}
  //wait move end
	this->m_statusGet.data=ANGLE_CHECK;
	int cntIsNewOmni=0;
  do{
  	//wait isNew
  	if(this->m_statusGetIn.isNew()){
  		cntIsNewOmni=0;
  		//read
  		this->m_statusGetIn.read();
  	}
  	else{cntIsNewOmni++;}
  }while(this->m_statusGet.data!=FINISH && cntIsNewOmni<10000);
  if(this->m_statusGet.data==FINISH){std::cout<<"finish!"<<std::endl;}
  if(cntIsNewOmni>=10000){std::cerr<<"status does not come!"<<std::endl;}

  std::cout<<"quit:9, continue:1"<<std::endl;
  int qflag=0;
  std::cin>>qflag;
  if(qflag==9){return RTC::RTC_ERROR;}
  //////// add 20110929 Toshiba ///////


  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PositionInput::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t PositionInput::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t PositionInput::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t PositionInput::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t PositionInput::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void PositionInputInit(RTC::Manager* manager)
  {
    RTC::Properties profile(positioninput_spec);
    manager->registerFactory(profile,
                             RTC::Create<PositionInput>,
                             RTC::Delete<PositionInput>);
  }
  
};



