#ifndef REFERENCE_HARDWARE_PARAMETERS__H
#define REFERENCE_HARDWARE_PARAMETERS__H 1

#include "svmlib.h"

/////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////

#define RS485_BAUDRATE (921600)

/////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////
//
// Parameters related to Wheeled BASE
//
// Base DOF Number
#define BASEDOFNUM (2)

// maximum velocity
int maxVelocityBase[2] = {10000, 10000};

// motor configuration
svm_config_t configBase[2] = {
  // see svm_config_t in svmlib.h
  //ver.no(Read only), lower,     upper, enc, drv, offset
  { 0,           -0x80000000,0x7fffffff,  -1,  -1, 0}, // DOF1/No.0
  { 0,           -0x80000000,0x7fffffff,   1,   1, 0}  // DOF2/No.1
};

// paramter settings
svm_param_t paramBase[2] = {
  // see svm_param_t in svmlib.h
  // P, D, I, VP,VD,VI, lower_veloc_limit, upper_veloc_limit, accel_limit, threshold 
  { 10, 0, 0, 10, 1, 0, -0x7fff, 0x7fff, 65000, 10}, // DOF1/No.0
  { 10, 0, 0, 10, 1, 0, -0x7fff, 0x7fff, 65000, 10}  // DOF2/No.1
};

/////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////
//
// Parameters related to ARM 
//
// ARM DOF Number including hand
#define ARMDOFNUM (7)
#define ARMDOFNUM_ARMONLY (ARMDOFNUM-1)

// motor configration
svm_config_t configArm[ARMDOFNUM] = {
  // see svm_config_t in svmlib.h
  //version no(Read only), lower, upper, enc, drv, offset
  { 0,-660000, 660000,   1,  1, 0},  // DOF1/No.0
  { 0,-509725, 983040,   1,  1, 0},  // DOF2/No.1
  { 0, -51200, 529066,  -1, -1, 0},  // DOF3/No.2
  { 0,-487667, 487666,   1,  1, 0},  // DOF4/No.3
  { 0,-316667, 450000,   1,  1, 0},  // DOF5/No.4
  { 0, -96712,  96711,   1,  1, 0},  // DOF6/No.5
  { 0,   2000, 444013,   1,  1, 0}   // DOF7/No.6 ベクトルセンサなしバージョン
//{ 0,  20000, 342425,  -1, -1, 0}   // DOF7/No.6 ベクトルセンサ付きバージョン
};

// paramter settings
svm_param_t paramArm[ARMDOFNUM] = {
  // see svm_param_t in svmlib.h
  // P, D, I, VP,VD,VI, lower_veloc_limit, upper_veloc_limit, threshold
  {  4, 0, 0, 6, 1, 0, -0x7fff, 0x7fff, 4000, 10},  // DOF1/No.0 new
  {  2, 1, 0, 7, 1, 0, -0x7fff, 0x7fff, 4000, 10},  // DOF2/No.1 new
  {  2, 1, 0, 6, 1, 0, -0x7fff, 0x7fff, 4000, 10},  // DOF3/No.2
  {  4, 3, 0, 7, 1, 0, -0x7fff, 0x7fff, 4000, 10},  // DOF4/No.3
  {  6, 1, 0,  7, 1, 0, -0x7fff, 0x7fff, 4000, 10},  // DOF5/No.4 new 
  { 10, 1, 1, 7, 1, 0, -0x7fff, 0x7fff, 4000, 10},  // DOF6/No.5 new
  { 30, 3, 1, 30, 3,0, -0x7fff, 0x7fff, 10000, 10}   // DOF7/No.6
};

double arm_mod_ratio[ARMDOFNUM] = { 360,    640,   300,   266,   300,   100,   2.2};
int    arm_enc_pulse[ARMDOFNUM] = {1000,   1024,  1024,  1000,  1000,   512,   512};
double    arm_enc_dt[ARMDOFNUM] = {800.0, 600.0, 600.0, 700.0, 500.0, 500.0, 600.0};

#endif
